#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/stat.h> //содержатся флаги режимов доступа к файлам (mode)
#include <fcntl.h>    //содержатся флаги открытия файлов (flags)
#include <unistd.h>
#include <signal.h>
#include <string.h>

/*
    КОДЫ ЗАВЕРШЕНИЯ РАБОТЫ
    3 - попытка деления на 0
    2 - есть 2 корня уравнения
    1 - есть 1 корень уравнения
    0 - нет корней уравнения
    -1 - нет пути к файлу
    -2 - не удалось найти файл
    -3 - не удалось прочитать файл
    -4 - не удалось записать в файл
    -5 - некорректные данные
*/
char *readFile(char *path)
{
    int flags = O_RDONLY;
    int mode = S_IRWXU | S_IRWXG | S_IRWXO;
    int fd = open(path, flags, mode); // Создание файлового дискриптора, для работы с файлом

    if (fd == -1)
    {
        printf("\nНе удалось открыть файл\n");
        exit(2);
    }
    int size_ = 1000;
    char *buffer = calloc(size_, sizeof(char)); // буфер для записи содержимого файла

    int numread = read(fd, buffer, 1000); // считывание файла buffer размером с size_

    if (numread < 0)
    {
        printf("\nНе удалось прочитать файл\n");
        exit(3);
    }

    close(fd); // закрытие файлового дискриптора

    return buffer;
}

int main(int argc, char *argv[])
{
    if (!argv[1] && !argv[2])
    {
        printf("\nНе удалось найти путь к файлу\n");
        exit(-1); // заканчиваем программу если путь к файлу пустой
    }

    char *pathname = argv[1];         // путь к файлу с аргументами
    const char *thisAnswer = argv[2]; // путь к файлу с ответом

    __pid_t childid;

    switch (childid = fork())
    {
    case -1:
    {
        printf("Не удалось запустить дочерний процесс\n");
        exit(-1);
        break;
    }
    case 0:
    {
        // запускаем 2 дочерний процесс для решения задачи
        const char *buffer = readFile(pathname);         // чтение данных из файла
        char *args4Child[] = {thisAnswer, buffer, NULL}; // аргументы для передачи в дочерний процесс

        execve("child", args4Child, NULL); // запускаем дочерний процесс

        printf("Не удалось запустить дочерний процесс\n");
        exit(-1);
        break;
    }
    default:
    {
        // ждем показ завершится 2 дочерний процесс
        int codeExit; //

        switch (wait(&codeExit))
        {
        case -1:
        {
            printf("Произошла ошибка вызова процесса для решения задачи");
            break;
        }
        default:
        {
            const char *buffer = readFile(thisAnswer);
            if(strlen(buffer) > 0)
            {
                printf("\nРОДИТЕЛЬСКИЙ ПРОЦЕСС\n");
                printf("%s",buffer);
            }
            break;
        }
        }
        break;
    }
    }
}